function [NStat, NSymb,  LFilt, REAL_COEFF] = dcv_chk (AR, AC, V, H);

%dcv_chk   Checks the parameters of a convolution model and returns its
%          dimensions as well as a flag indicating whther the model is
%	   real or complex.
%	Use: [NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,H).

% DCV Toolbox, version 1.1
% Olivier Capp, 23/07/97 - 23/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input args.
error(nargchk(4, 4, nargin));

% Check transition matrix
[NStat, NSymb] = size(AR);
[Nr, Nc] = size(AC);
if ((Nr ~= NSymb) | (Nc ~= NStat))
  error('Rows and columns of the transition matrix have incompatible sizes.');
end;

% State vectors
[LFilt, Nc] = size(V);
if (Nc ~= NStat)
  error('Wrong number of state vectors.')
end;

% Filter coefficients
[Nr, Nc] = size(H);
if (Nc ~= 1)
  error('Filter must be a column vector.');
end;
if (Nr ~= LFilt)
  error('Length of the filter is incorrect.');
end;

% Real of complex coefficients ?
if (isreal(V) & isreal(H))
  REAL_COEFF = 1;
else
  REAL_COEFF = 0;
end;
