function [X, symb, State1] = dcv_gen (v, H, sigma2, T);

%dcv_gen   Generates synthetic data for a convolution model.
%	Use: [X,symb, State1]=dcv_gen(v,H,sigma2,T). symb contains the emitted
%	symbols from 1 to T and State1 is the initial state vector (symb(1),
%	symb(0), ... symb(2-lenght(H))).
%	Note that the filter used is conj(H) so that the associated state-
%	space model indeed corresponds to H'*V.

% DCV Toolbox, version 1.1
% Olivier Capp, 24/07/97 - 26/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Needed functions
if (~((exist('randindx') == 2) | (exist('randindx') == 3)))
  error('Function randindx is missing.');
end;

% Input args.
error(nargchk(4, 4, nargin));
% Check input args.
NSymb = length(v);
LFilt = length(H);
[AR,AC,V] = dcv_init(v,LFilt);
NStat = length(V(1,:));

% Draw initial state (all states are equiprobables)
State1 = reshape(V(:,randindx(ones(1,NStat)/NStat, 1)),LFilt,1);
% Draw next symbols
symb = reshape(v(randindx(ones(1,NSymb)/NSymb,T-1)),T-1,1);

% Filter using conj(H) as a filter
X = filter(conj(H), 1, [flipud(State1);symb]);
% Discard the first (LFilt-1) elements (used for the inital condition)
X = X(LFilt:LFilt+T-1);

% Add noise
if (isreal(H) & isreal(v))
  % Real
  X = X + sqrt(sigma2)*randn(T,1);
else
  % Complex
  X = X + sqrt(sigma2/2)*(randn(T,1) + i*randn(T,1));;
end;

% Ouput
symb = [State1(1); symb];
