%xample_c  Illustration of the use of DCV functions for complex signals.

% DCV Toolbox, version 1.1
% Olivier Capp, 24/07/97 - 12/03/98
% ENST Dpt. Signal / CNRS URA 820, Paris

clear all;
i = sqrt(-1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1,'\nGenerating an observation sequence.\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Symbols
v = [(1+i); (1-i); (-1-i); (-1+i)];
%v = [1; i; -1; -i];
% Filter
H_s = [1.5+i*0.2; 0.3-i*0.9];
sigma2_s = sum(abs(H_s).^2)/10;		% 10 dB SNR

% Generate an observation sequence
T = 100;
X = dcv_gen(v,H_s,sigma2_s,T);

% Initial guess for the parameters
H_0 = [1+i*eps; 1+i*eps];
sigma2_0 = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate parameters using EM
fprintf(1,'\nEstimating the parameters using EM\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nit = 20;
[H,sigma2,logl] = dcv_em(X,v,H_0,sigma2_0,Nit);

% Look at the results
clf;
subplot(2,1,1);
plot((0:Nit),logl);
title('Log-likelihood');
subplot(2,2,3);
plot(H(1,1), 'o');
hold on;
plot(H(1,2:Nit+1).*(0.98+0.04*rand(1,Nit)), 'x');
plot(H_s(1), '*r');
title('H(0)');
subplot(2,2,4);
plot(H(2,1), 'o');
hold on;
plot(H(2,2:Nit+1).*(0.98+0.04*rand(1,Nit)), 'x');
plot(H_s(2), '*r');
title('H(1)');
drawnow;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bayesian analysis of the model
fprintf(1, '\nContinue for bayesian analysis of the model...\n');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nit = 400;
[H,sigma2] = dcv_baye(X,v,2,Nit);

% Plot
clf;
subplot(2,1,1);
plot(real(H)');
title('Real part of H(0) and H(1)');
subplot(2,1,2);
plot(imag(H)');
title('Imaginary part of H(0) and H(1)');

fprintf(1, '\nNote that you may end up with a multiplication by -1 or +/-i which is absolutely\nnormal because of the blind context and the fact that the set of symbols is\nunchanged by such a mutiplication (this is an identifiability problem, but there\nmay also be local minimas).\n' );
