%xample_r  Illustration of the use of DCV functions for real signals.

% DCV Toolbox, version 1.1
% Olivier Capp, 24/07/97 - 12/03/98
% ENST Dpt. Signal / CNRS URA 820, Paris

clear all;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1,'\nGenerating an observation sequence.\n');
fprintf(1,'This is a good example because the algorithm converges either\nto the MLE or to a local maximum depending on the noise! Modify\nthe initialization of the filter to H_0 = [0;1;0] to see the difference.\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Symbols
v = [-1; 1];
% Filter
H_s = [0.3; 1.5; -0.1];
sigma2_s = sum(H_s.^2)/10;	% 10 dB SNR
fprintf('Simulation: Filter Coeffs. [%.4f  %.4f  %.4f], Noise Var. %.4f.\n', H_s, sigma2_s);

% Generate an observation sequence
T = 150;
[X, symb_s] = dcv_gen(v, H_s, sigma2_s, T);

% Initial guess for the parameters
H_0 = [1;1;1];	
sigma2_0 = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate parameters using EM
fprintf(1, '\nEstimating the parameters with the EM algorithm.\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nit = 20;
[H,sigma2,logl] = dcv_em(X,v,H_0,sigma2_0,Nit);

% Look at the results
clf;
subplot(2,1,1);
plot((0:Nit), logl);
title('Log-likelihood');
subplot(2,1,2);
plot((0:Nit), H');
title('Filter coefficients');
drawnow;
fprintf('Estimation: Filter Coeffs. [%.4f  %.4f  %.4f], Noise Var. %.4f.\n', H(:,Nit+1), sigma2(Nit+1));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate parameters using Quasi-Newton optimization if optimization
% toolbox is present
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (exist('fminu') == 2)
  fprintf(1, '\nEstimating the parameters using Quasi-Newton optimization.\n');
  OPT = foptions;
  OPT(2) = 1e-2;	% With a short observation sequence it isn't worth
  OPT(3) = 1e-2;	% finding the MLE with a much better precision
  [H_qn, sigma2_qn, logl_qn] = dcv_qn(X, v, H_0, sigma2_0, OPT);
  fprintf('Estimation: Filter Coeffs. [%.4f  %.4f  %.4f], Noise Var. %.4f.\n', H_qn, sigma2_qn);
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate parameters using SAEM
fprintf(1, '\nContinue to estimate the parameters with the SAEM algorithm...\n');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nit = 80;
[H,sigma2,logl] = dcv_saem(X,v,H_0,sigma2_0,[20, (Nit-20)/2, (Nit-20)/2]);

% Look at the results
clf;
subplot(2,1,1);
plot((0:Nit), logl);
title('Log-likelihood');
subplot(2,1,2);
plot((0:Nit), H');
title('Filter coefficients');
fprintf('Estimation: Filter Coeffs. [%.4f  %.4f  %.4f], Noise Var. %.4f.\n', H(:,Nit+1), sigma2(Nit+1));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1, '\nIf the estimated filter looks OK, continue to estimate the symbols...\n');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[symb, State1, logl] = dcv_vit(X, v, H(:,Nit+1), sigma2(Nit+1));
ErrRate = (T-sum(symb == symb_s))/T
